%% ELEC-HRO FYT PD2a-08
% Design of Multiuser Non-Cooperative Communication Systems via Game Theory
% Name: WANG Zhen
% Student ID: 05711977
% Email: ee_wzx@stu.ust.hk
% Supervisor: Prof. Daniel P. Palomar

%% function "FYT.m"
% This is the main function for the simulation of the FYT project.
% FYT():        initialize the system by loading the system information
%               saved before. (default)
%               No argument is needed.
% FYT(Random):  initialize the system randomly.
%               Argument "Random" can be any number. ("1" recommended)

function FYT(Random)
%% Initialization
if nargin>=1 % random initialization
    [Nit Q N_T N_R H R_n P_max P_init] = Random_Init;
    save FYTsetting Nit Q N_T N_R H R_n P_max P_init;
    disp('Initialization: Random');
else % system information loaded from FYTsetting.mat
    try
        load FYTsetting.mat;
        disp('Initialization: system information loaded from FYTsetting.mat');
    catch
        [Nit Q N_T N_R H R_n P_max P_init] = Random_Init;
        save FYTsetting Nit Q N_T N_R H R_n P_max P_init;
        disp('File "FYTsetting.mat" cannot be found.');
        disp('Initialization: Random');
    end
end

%% Algorithms
% MIMO Sequential IWFA
Rate_Sequ = MIMO_Sequ_IWFA(Nit, Q, N_T, H, R_n, P_max, P_init);

% MIMO Simultaneous IWFA
Rate_Simu = MIMO_Simu_IWFA(Nit, Q, N_T, H, R_n, P_max, P_init);

% MIMO Partially Asynchronous IWFA
Total_B = 21;
Rate_Part_Asyn = cell(Total_B);
for B=1:2:Total_B % time constraint
    Rate_Part_Asyn{B} = MIMO_Part_Asyn_IWFA(Nit, Q, N_T, H, R_n, P_max, P_init, B);
end

%% Results
color=['y' 'k' 'b' 'm' 'r' 'c' 'g'];
n1 = 20;
% comparison between Sequ and Simu
figure
for i=1:Q
    plot(1:n1, Rate_Sequ(i,1:n1)', [color(mod(i, 7)+1) '--']);
    hold on
    plot(1:n1, Rate_Simu(i,1:n1)', color(mod(i, 7)+1));
end
V = axis;
axis([1 n1 V(3) V(4)])
title('Sequential vs Simultaneous IWFA');
xlabel('Iteration Index');
ylabel('Users\prime Rates');
legend('Sequential IWFA','Simultaneous IWFA','Location','SouthEast')

% comparison between Simu and Part_Asyn
figure
subplot(2,1,1);
plot(1:n1, Rate_Simu(1:Q,1:n1)');
V = axis;
axis([1 n1 V(3) V(4)])
title('Simultaneous IWFA');
xlabel('Iteration Index');
ylabel('Users\prime Rates');
subplot(2,1,2);
plot(1:n1, Rate_Part_Asyn{1}(1:Q,1:n1)');
V = axis;
axis([1 n1 V(3) V(4)])
title('Partially Asynchronous IWFA (time constraint: B = 1)');
xlabel('Iteration Index');
ylabel('Users\prime Rates');

n2 = 40;
% different constraints for Part_Asyn
figure
for B=1:2:11
    subplot(2,3,ceil(B/2));
    plot(1:n2, Rate_Part_Asyn{B}(1:Q,1:n2)');
    V = axis;
    axis([1 n2 V(3) V(4)])
    title(['time constraint: B = ' num2str(B)]);
    xlabel('Iteration Index');
    ylabel('Users\prime Rates');
    hold on
end

n3 = 60;
% comparison between Asyn and Part_Asyn
figure
for i=1:Q
    plot(1:n3, Rate_Part_Asyn{21}(i,1:n3)', [color(mod(i, 7)+1) '--']);
    hold on
    plot(1:n3, Rate_Part_Asyn{5}(i,1:n3)', color(mod(i, 7)+1));
end
V = axis;
axis([1 n3 V(3) V(4)])
title('Pseudo-Totally Asynchronous vs Partially Asynchronous IWFA');
xlabel('Iteration Index');
ylabel('Users\prime Rates');
legend('Pseudo-Totally Asynchronous IWFA (time constraint: B = 21)','Partially Asynchronous IWFA (time constraint: B = 5)','Location','SouthEast')

end